package muxer.filter;

import java.util.HashSet;
import java.util.Map;
import java.util.TreeMap;

import muxer.MuxFilter;
import muxer.MuxManager;

import common.StreamDescriptor;
import common.StreamIdentifier;
import common.packet.Packet;
import common.ts.ProgramAssociationTable;
import common.ts.ProgramInfo;

import demux.ts.TsInfo;
import demux.ts.TsRawDemuxer;
import demux.ts.TsRawPacketPart;

public class TsStreamIdentifierMuxFilter extends MuxFilter {
	private int scanSize; 
	private StreamIdentifier streamIdentifier=new StreamIdentifier();
	private int bytes;
	private HashSet streamFoundSet=new HashSet();
	
	private Map<Integer, StreamDescriptor> streamDescriptorMap;
	private int pmtId;
	private TsRawDemuxer demuxer;
	//private int programMapCount;
	
	public TsStreamIdentifierMuxFilter(int scanSize, TsRawDemuxer demuxer) {
		this.scanSize=scanSize;
		this.demuxer=demuxer;
	}
	
	public int addPacket(Packet packet) {
		final TsRawPacketPart tsRawPacketPart=(TsRawPacketPart)packet.getPayload();
		bytes += tsRawPacketPart.limit();
		final int offset=tsRawPacketPart.getTsSyncOffset();
		final TsInfo info=tsRawPacketPart.getTsInfo();

		if (streamDescriptorMap==null) {
			if (demuxer.getTableHandler().isFinished() && demuxer.getProgramInfo()!=null) {
				ProgramInfo programInfo[]=demuxer.getProgramInfo();
				for (int i=0;i<programInfo.length;i++) {
					if (programInfo[i].getStreamDescriptorMap()==null) {
						return MuxManager.DO_NOTHING;
					}
				}
				streamDescriptorMap=new TreeMap<Integer, StreamDescriptor>();
				for (int i=0;i<programInfo.length;i++) {
					Map<Integer, StreamDescriptor> sdm=programInfo[i].getStreamDescriptorMap();
					streamDescriptorMap.putAll(sdm);
				}
			}
		} else if (info.pidStart) {
			Integer pidId=Integer.valueOf(info.pid);
			StreamDescriptor streamDescriptor=(StreamDescriptor)streamDescriptorMap.get(pidId);
			if (streamDescriptor!=null && !streamFoundSet.contains(pidId)) {
				int ataptationOffset=(info.adaptationControl==0x30)? (tsRawPacketPart.get(offset+4) & 0xff) + 1 : 0;
				streamFoundSet.add(pidId);
				int mpegHeaderlen=tsRawPacketPart.get(offset + ataptationOffset + 12)&0xff;
				StreamDescriptor coreStreamDescriptor=streamDescriptor.getCoreStreamDescriptor();
				int streamId=coreStreamDescriptor==null?info.pid:coreStreamDescriptor.getStreamId();
				StreamDescriptor newStreamDescriptor = streamIdentifier.getStreamDescriptor(streamId, tsRawPacketPart, offset + ataptationOffset + 12 + 1+ mpegHeaderlen);
				
				if (newStreamDescriptor !=null) {
					newStreamDescriptor.setProgramNumber(streamDescriptor.getProgramNumber());
					if (coreStreamDescriptor==null) {
						streamDescriptorMap.put(pidId, newStreamDescriptor);
					} else {
						//This is a little bit of a hack, but because I don't know how to decode TrueHD or dts-HD info, it works nicely
						streamDescriptor.setCoreStreamDescriptor(newStreamDescriptor);
					}
				}
				return MuxManager.DO_NOTHING;
			}

		}
		if (bytes < scanSize) {
			return MuxManager.DO_NOTHING;
		} else {
			return MuxManager.ABORT;
		}
	}

	public Map<Integer, StreamDescriptor> getStreamDescriptorMap() {
		return streamDescriptorMap;
	}

	public void setStreamDescriptorMap(Map<Integer, StreamDescriptor> streamDescriptorMap) {
		this.streamDescriptorMap = streamDescriptorMap;
	}

	public int getPmtId() {
		return pmtId;
	}

	public void setPmtId(int pmtId) {
		this.pmtId = pmtId;
	}

	public int getPosition() {
		return POSITION_PRE_BUFFER;
	}
}
