package ui.tonmt;

import java.io.File;
import java.util.prefs.Preferences;

import common.Logger;

public class ToNMTPrefs {
	private Logger logger=Logger.getLogger(this.getClass());
	
	public static final String ACTION_CONVERT="c";
	public static final String ACTION_TRANSCODE="t";
	public static final String ACTION_PASSTHROUGH="p";
	public static final String ACTION_ADD_AC3="ac3";
	
	public static final String AUDIO_TYPE_AC3="AC3";
	public static final String AUDIO_TYPE_DTS="dts";
	public static final String AUDIO_TYPE_LPCM="LCPM";
	public static final String AUDIO_TYPE_TRUEHD="TrueHD";
	public static final String AUDIO_TYPE_EAC3="EAC3";
	
	public static final String COMPLETED_ACTION_NONE="None";
	public static final String COMPLETED_ACTION_EXIT="Exit";
	public static final String COMPLETED_ACTION_PROCESS="Run Process";
	
//	private static final String H264INFO_PATH="h264info";
	private static final String EAC3TO_PATH="eac3to";
	private static final String TSMUXER_PATH="tsMuxer";
	
	private static final String DEMUX_PATH_NAME="demuxPathName";
	private static final String CONVERT_TO_TYPE="convertToType";
	
	private static final String COMPLETED_ACTION="completedAction";
	private static final String COMPLETED_PROCESS="completedProcess";
	
	private static final String ENSURE_SD_AUDIO="ensureSDAudio";
	
	private static final String KEEP_DIALOG_NORMALIZATION="keepDialnorm";
	
	private Preferences prefs =  Preferences.userRoot().node("/ToNMT");

	private static ToNMTPrefs cache;
	
	private ToNMTPrefs() {};
	
	public static ToNMTPrefs getPrefs() {
		if (cache==null) {
			cache=new ToNMTPrefs();
		}
		return cache;
	}
	
	public String getDemuxPathName() {
		return prefs.get(DEMUX_PATH_NAME, "C:\\RemuxTool");
	}
	public String getDemuxPrefix() {
		return getDemuxPathName() + File.separator + "HDDVD";
	}
	public String getConvertToType() {
		return prefs.get(CONVERT_TO_TYPE, AUDIO_TYPE_AC3);
	}
	public String getEac3toPath() {
		return prefs.get(EAC3TO_PATH, ".\\eac3to\\eac3to.exe");
	}
	public String getTsMuxerPath() {
		return prefs.get(TSMUXER_PATH, ".\\tsMuxeR\\tsMuxeR.exe");
	}
	public String getCompletedAction() {
		return prefs.get(COMPLETED_ACTION, COMPLETED_ACTION_NONE);
	}
	public String getCompletedProcess() {
		return prefs.get(COMPLETED_PROCESS, "shutdown /s");
	}
	public void setDemuxPathName(String pathName) {
		prefs.put(ToNMTPrefs.DEMUX_PATH_NAME, pathName);
	}
	
	public void setConvertToType(String type) {
		prefs.put(ToNMTPrefs.CONVERT_TO_TYPE, type);
	}
	public void setEac3toPath(String path) {
		prefs.put(ToNMTPrefs.EAC3TO_PATH, path);
	}
	public void setTsMuxerPath(String path) {
		prefs.put(ToNMTPrefs.TSMUXER_PATH, path);
	}
	
	public String getTrueHDAction() {
		return prefs.get(AUDIO_TYPE_TRUEHD, ACTION_CONVERT);
	}
	public String getEAC3Action() {
		return prefs.get(AUDIO_TYPE_EAC3, ACTION_CONVERT);
	}
	public String getDtsAction() {
		return prefs.get(AUDIO_TYPE_DTS, ACTION_PASSTHROUGH);
	}
	
	public boolean isCore() {
		return prefs.getBoolean("core", true);
	}
	
	public boolean isEnsureSDAudio() {
		return prefs.getBoolean(ENSURE_SD_AUDIO, false);
	}
	
	public void setEnsureSDAudio(boolean flag) {
		prefs.putBoolean(ENSURE_SD_AUDIO, flag);
	}
	public boolean isKeepDialogNormalization() {
		return prefs.getBoolean(KEEP_DIALOG_NORMALIZATION, true);
	}
	public void setKeepDialogNormalization(boolean flag) {
		prefs.putBoolean(KEEP_DIALOG_NORMALIZATION, flag);
	}
	public void setTrueHDAction(String action) {
		prefs.put(AUDIO_TYPE_TRUEHD, action);
	}
	public void setEAC3Action(String action) {
		prefs.put(AUDIO_TYPE_EAC3, action);
	}
	public void setDtsAction(String action) {
		prefs.put(AUDIO_TYPE_DTS, action);
	}
	public void setCore(boolean flag) {
		prefs.putBoolean("core", flag);
	}
	public void setCompletedAction(String action) {
		prefs.put(COMPLETED_ACTION, action);
	}
	public void setCompletedProcess(String process) {
		prefs.put(COMPLETED_PROCESS, process);
	}
	
	
	//Since the AC3 stream is currently unusable, make it small
	public int getTrueHdCoreRate() {
		return 192;
	}
	
	public void preferenceCheck() {
		ToNMTPrefs prefs=ToNMTPrefs.getPrefs();
		
		File eac3toFile=new File(prefs.getEac3toPath());
		if (!eac3toFile.canExecute()) {
			logger.warn("Could not find EAC3TO at: " + eac3toFile.getAbsolutePath());
		}
		File tsMuxerFile=new File(prefs.getTsMuxerPath());
		if (!tsMuxerFile.canExecute()) {
			logger.warn("Could not find TsMuxeR at: " + tsMuxerFile.getAbsolutePath());
		}
		logger.info("Will convert to " + prefs.getConvertToType()+ " as necessary");
		String demuxPathName=prefs.getDemuxPrefix();
		File demuxFilePath=new File(demuxPathName).getParentFile();
		if (demuxFilePath.canWrite()) {
			logger.info("Using " + demuxPathName+ " for temp files");
		} else {
			logger.error("Cannot write to temp space: " +demuxPathName);
		}
		if (prefs.isEnsureSDAudio() && prefs.getConvertToType().equals(AUDIO_TYPE_LPCM)) {
			logger.info("LPCM & Ensure SD Audio selected.  AC3 will be used for primary audio.");
		}
	}

}
