package util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.WritableByteChannel;

import ui.CliJobListener;

import common.BitStream;
import common.BufferedWritableByteChannel;
import common.DirectMemoryManager;
import common.Job;
import common.Util;
import common.avc.AvcPulldown;
import common.mpeg.MpegUtil;

public class H264ConvJ {
	private static final int READ_SIZE=20*1024*1024;
	private WritableByteChannel out;
	private AvcPulldown avcPulldown=new AvcPulldown();
	
	public static void main(String args[]) {
		long start=System.currentTimeMillis();
		try {
			System.out.println("H264ConvJ - HD DVD to BluRay compliant VC1 processor");
			if (args.length!=2) {
				System.out.println("Usage:");
				System.out.println("H264ConvJ inH264File outH264File");
				System.exit(-1);
				//args=new String[]{"f:/Bourne_fd55.vc1","C:/my_Bourne2_fd55.vc1"};
			}
			H264ConvJ test=new H264ConvJ(args[1]);
			System.out.println("Processing " + args[0]);
			test.parse(args[0]);
			System.out.println();
			System.out.println("Completed in " + Util.timeToString(System.currentTimeMillis()-start));
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}
	public H264ConvJ(String fileName) throws IOException {
		//out=new FileOutputStream(new File(fileName)).getChannel();
		out=new BufferedWritableByteChannel(new File(fileName),10*1024*1024);
	}	
	public void parse(String fileName) throws IOException {
		long filePosition=0;
		FileChannel fc=new FileInputStream(fileName).getChannel();
		Job job=new Job(new CliJobListener());
		job.setSize((int)(fc.size()/Job.KB));
		ByteBuffer byteBuffer=ByteBuffer.allocate(0);//Dummy
		boolean lastPacket=false;
		int sequenceStart=-1;
		int writeStart=0;
		while (!lastPacket) {
			filePosition+=byteBuffer.position();
			job.setPosition((int)(filePosition/Job.KB));
			int readSize=(int)Math.min(READ_SIZE, fc.size() - filePosition);
			lastPacket=readSize!=READ_SIZE;
			byteBuffer=DirectMemoryManager.mapMemory(fc, FileChannel.MapMode.READ_ONLY, filePosition, readSize);
			writeStart=0;
			sequenceStart=-1;
			for (int start=0;start>=0;) {
				start=MpegUtil.findNextStartCode(byteBuffer, start);
				if (start>=0) {
					int nextType=byteBuffer.get(start+3) & 0x1f;
					if (nextType==7) {
						start +=4;//Copy the header
						byteBuffer.position(writeStart);
						byteBuffer.limit(start);
						out.write(byteBuffer);
						byteBuffer.limit(byteBuffer.capacity());
						sequenceStart=start;
					} else if (sequenceStart != -1) {
						byteBuffer.position(sequenceStart);
						byteBuffer.limit(start);
						BitStream bitStream=new BitStream(byteBuffer.remaining());
						avcPulldown.copySequence(byteBuffer, sequenceStart, bitStream);
						bitStream.write(out);
						writeStart=start;
						byteBuffer.limit(byteBuffer.capacity());
						sequenceStart = -1;
					}
					start+=4;
				}
			}
		}
		byteBuffer.position(writeStart);
		out.write(byteBuffer);
		out.close();
		fc.close();		
	}
}
