package util;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import muxer.MuxManager;
import muxer.filter.RaggedRawStreamMuxFilter;
import muxer.filter.TsRawToPesMuxFilter;
import muxer.ts.TsPesMuxer;
import muxer.ts.TsRawMuxer;
import ui.CliJobListener;

import common.BufferedWritableByteChannel;
import common.FileStream;
import common.Job;
import common.Logger;
import common.Util;
import common.btv.TpParser;
import common.ts.ChannelInfo;
import common.ts.MasterGuideTable;
import common.ts.ProgramAssociationTable;
import common.ts.ProgramMapTable;
import common.ts.TableHandler;
import common.ts.TerrestrialVirtualChannelTable;
import common.ts.TsConstants;
import common.ts.TsTable;
import common.ts.TsTableParsedListener;

import demux.ts.TsRawDemuxer;
import demux.ts.TsRawPacketPart;

public class StreamSnipJ implements TsTableParsedListener {
	private static final String KEEP_ORIGINAL="-k";
	private static final String BLURAY_FORMAT="-b";
	private HashMap<TsTable, TsRawPacketPart[]> parseMap = new HashMap<TsTable, TsRawPacketPart[]>();
	
	private Logger logger=Logger.getLogger(this.getClass());
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length==0 || args.length > 3) {
			System.out.println("StreamSnipJ v1.3");
			System.out.println("Usage:");
			System.out.println("StreamSnipJ [-k] [-p] fileName");
			System.out.println("-k=keep original file");
			System.out.println("-b=BluRay (PS3) format");
			System.exit(-1);
		}
		boolean keepOriginal=false,bluRayFormat=false;
		for (int i=0;i<args.length - 1;i++) {
			if (args[i].equals(KEEP_ORIGINAL)) {
				keepOriginal=true;
			} else if (args[i].equals(BLURAY_FORMAT)) {
				bluRayFormat=true;
			} 
		}
		File inFile=new File(args[args.length-1]);
		if (!inFile.exists()) {
			System.out.println("File not found: " + inFile.getAbsolutePath());
			System.exit(-1);
		}
		File outFile=new File(inFile.getAbsolutePath() + ".wrk");
		//File outFile=new File("C:/test.ts");
		//String titleName=args.length==0 ? "M:/tools/My Name Is Earl-(Stole a Motorcycle)-2008-04-10-0.ts" : args[0];
		//String outFileName="C:/test.ts";
		boolean writable=inFile.canWrite();
		try {	
			if (new StreamSnipJ().snip(inFile, outFile, bluRayFormat)) {
				inFile.setWritable(true);
				if (keepOriginal) {
					inFile.renameTo(new File(inFile.getAbsolutePath()+".bak"));
					outFile.renameTo(inFile);
				} else {
					//Deal with mapped file bug
					long end=System.currentTimeMillis() + 30000;
					boolean deleted;
					while ((!(deleted=inFile.delete())) && System.currentTimeMillis() < end) {
						System.gc();
						Thread.yield();
					}
					if (deleted) {
						outFile.renameTo(inFile);
					} else {
						System.out.println("WARNING: Failed to delete source file");
					}
				}
				inFile.setWritable(writable);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	private boolean snip(File inFile, File outFile, boolean bluRayFormat) throws IOException {
		long start=System.currentTimeMillis();
		CliJobListener jobListener =new CliJobListener();

		FileStream fileJob=new FileStream(new Job(jobListener), new File[]{inFile});
		TsRawDemuxer demuxer=new TsRawDemuxer();
		TableHandler tableHandler=demuxer.getTableHandler();
		MasterGuideTable mgt=new MasterGuideTable();
		tableHandler.add(mgt);
		TerrestrialVirtualChannelTable tvct=new TerrestrialVirtualChannelTable();
		tableHandler.add(tvct);
		tvct.addListener(this);
		tableHandler.getPat().addListener(this);

		MuxManager muxManager=new MuxManager(demuxer);
		
		TpParser tpParser=new TpParser(inFile);
		if (tpParser.isSnipped()) {
			logger.warn("File appears to have already been snipped.  Exiting.");
			return false;
		}
		logger.debug("BTV Info:");
		Map btvMap=tpParser.parse();
		for (Iterator i=btvMap.entrySet().iterator();i.hasNext();) {
			Map.Entry entry=(Map.Entry)i.next();
			if (logger.isDebug()) {
				logger.debug(entry.getKey() + "=" + entry.getValue());
			}
		}
		ByteBuffer metaBuffer=tpParser.getMetaBuffer();
		String uniqueChannelId=(String)btvMap.get(TpParser.UNIQUE_CHANNEL_ID);
		int majorChannel=Integer.parseInt(uniqueChannelId.substring(0, 2));
		int minorChannel=Integer.parseInt(uniqueChannelId.substring(2, 4));
		/*int majorChannel=9;
		int minorChannel=1;*/
		
		Map streamMap=muxManager.getStreamDescriptorMap(fileJob);
		ChannelInfo channelInfo[]=tvct.getChannelInfo();
		
		logger.info("Channels:");
		short programNumber=Short.MIN_VALUE;
		
		for (int i=0;i<channelInfo.length;i++) {
			if (channelInfo[i].getMajorChannel()==majorChannel && channelInfo[i].getMinorChannel()==minorChannel) {
				programNumber=channelInfo[i].getProgramNumber();
				logger.info(channelInfo[i] + "*");
			} else {
				logger.info(channelInfo[i]);
			}
		}
		if (programNumber != Short.MIN_VALUE) {
			demuxer.setSelectedProgramNumber(programNumber);
			tableHandler.remove(mgt);
			tableHandler.remove(tvct);
			if (bluRayFormat) {
				TsPesMuxer muxer=new TsPesMuxer(new BufferedWritableByteChannel(outFile, 10*1024*1024));
				muxManager.add(new TsRawToPesMuxFilter());
				muxManager.add(muxer);
			} else {
				TsRawPacketPart headers[][]=new TsRawPacketPart[3][];
				ProgramAssociationTable pat=tableHandler.getPat();
				ProgramMapTable pmt=(ProgramMapTable)tableHandler.getTables(ProgramMapTable.class)[0];
				headers[0]=pat.filter(false);
				headers[1]=pmt.filter(false);
				TsRawPacketPart tvctBuffer[]=parseMap.get(tvct);
				for (int i=0;i<tvctBuffer.length;i++) {
					if (tvctBuffer[i].getTsInfo().pid != TsConstants.ATSC_TABLE_PID) {
						tvctBuffer[i].setReadWrite();
						ByteBuffer byteBuffer=tvctBuffer[i].getByteBuffer();
						byteBuffer.position(tvctBuffer[i].getTsSyncOffset() + 1);
						byteBuffer.putShort((short)(0x400 + TsConstants.ATSC_TABLE_PID));
						tvctBuffer[i].setByteBuffer(byteBuffer);//Resets TsInfo
					}
				}
				//Hack
				tvct.setSelectedProgramNumbers(new short[]{programNumber});
				headers[2]=tvct.filter(true);
				
				muxManager.add(new RaggedRawStreamMuxFilter(headers));
				TsRawMuxer muxer=new TsRawMuxer(outFile);
				muxManager.add(muxer);
			}
			fileJob.reset();

			fileJob.setStreamSize(fileJob.getStreamSize() - metaBuffer.capacity() - TsConstants.TS_PACKET_SIZE);

			muxManager.mux(fileJob);
			System.out.println();
			//Write the BTV Meta back
			FileChannel fc=new RandomAccessFile(outFile,"rw").getChannel();
			fc.position(fc.size());
			metaBuffer.rewind();
			fc.write(metaBuffer);
			fc.close();
			
			long origLen=inFile.length();
			long finalLen=outFile.length();
			logger.info("Shrunk " + new BigDecimal((origLen-finalLen)*100).divide(new BigDecimal(origLen),2,BigDecimal.ROUND_HALF_UP) + "% in " + 
					Util.timeToString(System.currentTimeMillis()-start));
			logger.info("Bad packets: " + demuxer.getBadPacketCount() + " of " + demuxer.getPacketCount() + " - " + 
					new BigDecimal(demuxer.getBadPacketCount()*100).divide(new BigDecimal(demuxer.getPacketCount()), 3, BigDecimal.ROUND_HALF_UP) + "%");
			
			return true;
		} else {
			logger.error("Could not find channel reported by BTV in stream");
			return false;
		}
	}
	public void tsTableDidParse(TsTable tsTable, TsRawPacketPart buffer[]) {
		if (tsTable instanceof ProgramAssociationTable) {
			//This should work because the pat listens to itself and should have added these
			ProgramAssociationTable pat=(ProgramAssociationTable)tsTable;
			ProgramMapTable pmtArray[]=(ProgramMapTable[])pat.getTableHandler().getTables(ProgramMapTable.class);
			for (int i=0;i<pmtArray.length;i++) {
				pmtArray[i].addListener(this);
			}
		}
		parseMap.put(tsTable, buffer);
	}
}
